@@ -8,7 +8,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
+ * as published by the Free Software Foundation; either version 3 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -16,10 +16,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -27,12 +25,12 @@
 #include "gnutls_errors.h"
 #include "gnutls_auth.h"
 #include "gnutls_auth.h"
-#include "gnutls_algorithms.h"
-#include "auth_cert.h"
-#include "auth_psk.h"
+#include "algorithms.h"
+#include <auth/cert.h>
+#include <auth/psk.h>
+#include <auth/anon.h>
 #include <gnutls_datum.h>
 
-#include "auth_anon.h"
 /* The functions here are used in order for authentication algorithms
  * to be able to retrieve the needed credentials eg public and private
  * key etc.
@@ -92,8 +90,8 @@ gnutls_credentials_clear (gnutls_session_t session)
  * For %GNUTLS_CRD_CERTIFICATE, @cred should be
  * #gnutls_certificate_credentials_t.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS (zero) is returned,
- *   otherwise an error code is returned.
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
+ *   otherwise a negative error code is returned.
  **/
 int
 gnutls_credentials_set (gnutls_session_t session,
@@ -301,6 +299,7 @@ void
 _gnutls_free_auth_info (gnutls_session_t session)
 {
   dh_info_st *dh_info;
+  rsa_info_st *rsa_info;
 
   if (session == NULL || session->key == NULL)
     {
@@ -343,6 +342,7 @@ _gnutls_free_auth_info (gnutls_session_t session)
           break;
 
         dh_info = &info->dh;
+        rsa_info = &info->rsa_export;
         for (i = 0; i < info->ncerts; i++)
           {
             _gnutls_free_datum (&info->raw_certificate_list[i]);
@@ -353,6 +353,7 @@ _gnutls_free_auth_info (gnutls_session_t session)
         info->ncerts = 0;
 
         _gnutls_free_dh_info (dh_info);
+        _gnutls_free_rsa_info (rsa_info);
       }
 
 
