@@ -4,21 +4,19 @@
  * 
  * Author: Nikos Mavrogiannopoulos
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * The GnuTLS is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- * MA 02111-1307, USA
-*/
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ */
 
 #include <gnutls_int.h>
 #include <gnutls/pkcs11.h>
@@ -62,7 +60,7 @@ struct gnutls_privkey_st
  * actually the type of the subsystem used to set this private key.
  *
  * Returns: a member of the #gnutls_privkey_type_t enumeration on
- *   success, or a negative value on error.
+ *   success, or a negative error code on error.
  **/
 gnutls_privkey_type_t
 gnutls_privkey_get_type (gnutls_privkey_t key)
@@ -80,7 +78,7 @@ gnutls_privkey_get_type (gnutls_privkey_t key)
  * the security parameter of the key.
  *
  * Returns: a member of the #gnutls_pk_algorithm_t enumeration on
- *   success, or a negative value on error.
+ *   success, or a negative error code on error.
  **/
 int
 gnutls_privkey_get_pk_algorithm (gnutls_privkey_t key, unsigned int *bits)
@@ -246,7 +244,7 @@ _gnutls_privkey_get_public_mpis (gnutls_privkey_t key,
  *
  * This function will initialize an private key structure.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
 int
@@ -271,6 +269,8 @@ gnutls_privkey_init (gnutls_privkey_t * key)
 void
 gnutls_privkey_deinit (gnutls_privkey_t key)
 {
+  if (key == NULL) return;
+
   if (key->flags & GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE)
     switch (key->type)
       {
@@ -315,7 +315,7 @@ static int check_if_clean(gnutls_privkey_t key)
  * The #gnutls_pkcs11_privkey_t object must not be deallocated
  * during the lifetime of this structure.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
 int
@@ -353,7 +353,7 @@ int ret;
  * The #gnutls_x509_privkey_t object must not be deallocated
  * during the lifetime of this structure.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
 int
@@ -391,7 +391,7 @@ int ret;
  * during the lifetime of this structure. The subkey set as
  * preferred will be used, or the master key otherwise.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  **/
 int
@@ -449,7 +449,7 @@ uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
  * Use gnutls_pubkey_get_preferred_hash_algorithm() to determine
  * the hash algorithm.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  * negative error value.
  *
  * Since: 2.12.0
@@ -510,7 +510,7 @@ cleanup:
  * Use gnutls_pubkey_get_preferred_hash_algorithm() to determine
  * the hash algorithm.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  *
  * Since: 2.12.0
@@ -564,7 +564,7 @@ cleanup:
  * This function will sign the given data using a signature algorithm
  * supported by the private key.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  * negative error value.
  -*/
 int
@@ -604,7 +604,7 @@ _gnutls_privkey_sign_hash (gnutls_privkey_t key,
  * This function will decrypt the given data using the algorithm
  * supported by the private key.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  * negative error value.
  **/
 int
