@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
+ * as published by the Free Software Foundation; either version 3 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -15,10 +15,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -59,6 +57,9 @@ system_errno (gnutls_transport_ptr p)
     case WSAEWOULDBLOCK:
       ret = EAGAIN;
       break;
+    case NO_ERROR:
+      ret = 0;
+      break;
     case WSAEINTR:
       ret = EINTR;
       break;
@@ -78,13 +79,13 @@ system_write (gnutls_transport_ptr ptr, const void *data, size_t data_size)
 }
 #else /* POSIX */
 int
-system_errno (gnutls_transport_ptr ptr)
+system_errno (gnutls_transport_ptr_t ptr)
 {
   return errno;
 }
 
 ssize_t
-system_writev (gnutls_transport_ptr ptr, const giovec_t * iovec,
+system_writev (gnutls_transport_ptr_t ptr, const giovec_t * iovec,
                int iovec_cnt)
 {
   return writev (GNUTLS_POINTER_TO_INT (ptr), (struct iovec *) iovec,
@@ -94,17 +95,47 @@ system_writev (gnutls_transport_ptr ptr, const giovec_t * iovec,
 #endif
 
 ssize_t
-system_read (gnutls_transport_ptr ptr, void *data, size_t data_size)
+system_read (gnutls_transport_ptr_t ptr, void *data, size_t data_size)
 {
   return recv (GNUTLS_POINTER_TO_INT (ptr), data, data_size, 0);
 }
 
 ssize_t
-system_read_peek (gnutls_transport_ptr ptr, void *data, size_t data_size)
+system_read_peek (gnutls_transport_ptr_t ptr, void *data, size_t data_size)
 {
   return recv (GNUTLS_POINTER_TO_INT (ptr), data, data_size, MSG_PEEK);
 }
 
+/* Wait for data to be received within a timeout period in milliseconds.
+ * If data_size > 0 it will return the specified amount of data in
+ * peek mode.
+ *
+ * Returns -1 on error, 0 on timeout.
+ */
+int system_recv_timeout(gnutls_transport_ptr_t ptr, unsigned int ms)
+{
+fd_set rfds;
+struct timeval tv;
+int ret, ret2;
+
+  FD_ZERO(&rfds);
+  FD_SET(GNUTLS_POINTER_TO_INT(ptr), &rfds);
+  
+  tv.tv_sec = 0;
+  tv.tv_usec = ms * 1000;
+  
+  ret = select(GNUTLS_POINTER_TO_INT(ptr)+1, &rfds, NULL, NULL, &tv);
+  if (ret <= 0)
+    return ret;
+
+  
+  ret2 = recv(GNUTLS_POINTER_TO_INT(ptr), NULL, 0, MSG_PEEK);
+  if (ret2 == -1)
+    return ret2;
+      
+  return ret;
+}
+
 /* Thread stuff */
 
 #ifdef HAVE_WIN32_LOCKS
@@ -149,6 +180,14 @@ gnutls_system_mutex_unlock (void **priv)
   return 0;
 }
 
+int
+_gnutls_atfork (void (*prepare) (void), void (*parent) (void),
+                void (*child) (void))
+{
+  return 0;
+}
+
+
 #endif /* WIN32_LOCKS */
 
 #ifdef HAVE_PTHREAD_LOCKS
@@ -207,6 +246,13 @@ gnutls_system_mutex_unlock (void **priv)
   return 0;
 }
 
+int
+_gnutls_atfork (void (*prepare) (void), void (*parent) (void),
+                void (*child) (void))
+{
+  return pthread_atfork (prepare, parent, child);
+}
+
 #endif /* PTHREAD_LOCKS */
 
 #ifdef HAVE_NO_LOCKS
@@ -235,6 +281,13 @@ gnutls_system_mutex_unlock (void **priv)
   return 0;
 }
 
+int
+_gnutls_atfork (void (*prepare) (void), void (*parent) (void),
+                void (*child) (void))
+{
+  return 0;
+}
+
 #endif /* NO_LOCKS */
 
 gnutls_time_func gnutls_time = time;
